#
# coursebuilder
#

from typing import Any
from argparse import ArgumentParser
from pathlib import Path
import yaml

from coursebuilder.schema import Schema


class LanguageSelector:
    context_lang: str


class Course:
    i18n_name: str = "lang.{}.yaml"
    mod_name: str = "mod.yaml"

    def __init__(self, *, path: Path) -> None:
        with open(path) as f:
            # resolve rest of bundle
            self.__data = yaml.load(f, Loader=yaml.Loader)
            # load i18n overlays
            self.__i18n = {
                f"{str(p).split('.')[1]}": yaml.load(open(p), Loader=yaml.Loader)
                for p in path.parent.glob(Course.i18n_name.format("*"))
            }

    def validate(self, *, schema: Schema, lang: str) -> None:
        print(self.__data)
        pass

    def __getitem__(self, name: str, /) -> Any:
        return (
            self.__data[name]
            if name in self.__data.keys()
            else self.__i18n[LanguageSelector.context_lang][name]
        )

    def __getattr__(self, name: str, /) -> Any:
        return self.__data[name]

    def __str__(self):
        return f"data:{self.__data}\ni18n:{self.__i18n}"


class StudyCourse:
    def __init__(self, *, path: Path) -> None:
        self.path = path

        with open(path) as f:
            self.__data = yaml.load(f, Loader=yaml.Loader)

        self.courses: dict[str, Course] = {
            f"{c}": Course(path=self.path.parent / c / Course.mod_name)
            for c in self.__data["courses"]
        }

    def __getattr__(self, name: str, /) -> Any:
        return self.__data[name] if self.__data else None

    def __str__(self):
        return f"path: {self.path}\ndata: {self.__data}"


def main():
    # parser
    parser = ArgumentParser(description="versatile curricula generator")

    # loading mode for internal database
    parser.add_argument(
        "-i",
        "--input",
        type=str,
        help="course file with definition of the course",
    )

    parser.add_argument(
        "-s",
        "--schema",
        type=str,
        help="schema to validate against",
    )

    # get arguments
    args = parser.parse_args()

    # just input
    if args.input and args.schema:
        with open(args.schema) as f_schema:
            schema = Schema(schema=yaml.load(f_schema, Loader=yaml.Loader))
            sc = StudyCourse(path=(Path(".") / args.input).absolute())

            LanguageSelector.context_lang = "de"

            for k in schema.keys():
                print(k)
                for shortcode, course in sc.courses.items():
                    print(course[k])


# run as main
if __name__ == "__main__":
    main()
